﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using HeuristicLab.Algorithms.MemPR.Permutation.LocalSearch;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.MemPR.Permutation {
  [Item("Exhaustive Inversion Improver", "")]
  [StorableClass]
  public class ExhaustiveInversionImprover : SingleSuccessorOperator, IImprovementOperator {
    #region Parameter properties
    public ILookupParameter<DoubleMatrix> DistanceMatrixParameter {
      get { return (ILookupParameter<DoubleMatrix>)Parameters["DistanceMatrix"]; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<Encodings.PermutationEncoding.Permutation> PermutationParameter {
      get { return (ILookupParameter<Encodings.PermutationEncoding.Permutation>)Parameters["Permutation"]; }
    }
    public ILookupParameter<IntValue> LocalEvaluatedSolutionsParameter {
      get { return (ILookupParameter<IntValue>)Parameters["LocalEvaluatedSolutions"]; }
    }
    #endregion

    [StorableConstructor]
    private ExhaustiveInversionImprover(bool deserializing) : base(deserializing) { }
    private ExhaustiveInversionImprover(ExhaustiveInversionImprover original, Cloner cloner) : base(original, cloner) { }
    public ExhaustiveInversionImprover()
      : base() {
      #region Create parameters
      Parameters.Add(new LookupParameter<DoubleMatrix>("DistanceMatrix", "The matrix which contains the distances between the cities."));
      Parameters.Add(new LookupParameter<IRandom>("Random", "A pseudo random number generator."));
      Parameters.Add(new ValueLookupParameter<Encodings.PermutationEncoding.Permutation>("Permutation", "The solution to be improved. This parameter is used for name translation only."));
      Parameters.Add(new LookupParameter<IntValue>("LocalEvaluatedSolutions", "The amount of locally evaluated solutions."));
      #endregion
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExhaustiveInversionImprover(this, cloner);
    }

    public override IOperation Apply() {
      var random = RandomParameter.ActualValue;
      var perm = PermutationParameter.ActualValue;

      var distances = DistanceMatrixParameter.ActualValue;
      Func<Encodings.PermutationEncoding.Permutation, CancellationToken, double> eval = (p, _) => {
        double length = 0;
        for (int i = 0; i < p.Length - 1; i++)
          length += distances[p[i], p[i + 1]];
        length += distances[p[p.Length - 1], p[0]];
        return length;
      };
      var quality = eval(perm, CancellationToken.None);
      var path = Exhaustive2Opt.HillClimb(random, perm, quality, false, eval, CancellationToken.None);
      var final = path.Last();

      LocalEvaluatedSolutionsParameter.ActualValue = new IntValue(final.Item3);

      return base.Apply();
    }
  }
}
