﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Core;
using System;
using System.Collections.Generic;

namespace HeuristicLab.Analysis.FitnessLandscape {
  public static class IListExtensions {
    public static void DuplicateLast<T>(this IList<T> l) {
      l.Add(l[l.Count - 1]);
    }

    public static void Shuffle<T>(this IList<T> list, IRandom random) {
      int n = list.Count - 1;
      while (n > 0) {
        int k = random.Next(n + 1);
        T value = list[k];
        list[k] = list[n];
        list[n] = value;
        n--;
      }
    }
  }

  public static class IDictionaryExtensions {
    public static TValue GetValueOrDefault<TKey, TValue>(this IDictionary<TKey, TValue> d, TKey key, Func<TValue> @default) {
      TValue value;
      return d.TryGetValue(key, out value) ? value : @default();
    }

    public static TValue GetValueOrDefault<TKey, TValue>(this IDictionary<TKey, TValue> d, TKey key, TValue @default = default(TValue)) {
      TValue value;
      return d.TryGetValue(key, out value) ? value : @default;
    }
  }
}
